var mssql = require('mssql');

var varTypeMap = {
    bit: "number",
    int: "number",
    smallint: "number",
    tinyint: "number",
    numeric: "number",
    decimal: "number",
    money: "number",
    smallmoney: "number",
    float: "number",
    real: "number",
    datetime: "string",
    timestamp: "string",
    char: "string",
    varchar: "string",
    text: "string",
    nchar: "string",
    nvarchar: "string",
    ntext: "string",
    binary: "string",
    varbinary: "string",
    image: "string"
};

module.exports = {
    testConnection: function(dbConfig, cb) {
        dbConfig.server = dbConfig.host;
        const pool = new mssql.ConnectionPool(dbConfig);

        pool.connect(err => {
            if (err) {
                pool.close();
                return cb(null, err.message);
            }
            pool.close();
            return cb(null, true);
        });
    },
    testSql: function(dbConfig, sql, params, cb) {
        dbConfig.server = dbConfig.host;
        var orginalSql = sql;

        // filter result
        sql = 'SELECT * FROM (' + sql + ') A WHERE 1 = 0';
        const pool = new mssql.ConnectionPool(dbConfig);

        pool.connect(err => {
            if (err) {
                pool.close();
                return cb(null, {
                    success: false,
                    message: err.message
                });
            }
            var req = pool.request();

            // input parameter
            var i = 0;
            sql = sql.replace(/\?/g, function(id) {
                var pname = '@param' + i;
                req.input('param' + i, params[i]);
                i++;
                return pname;
            });
            
            req.query(sql, (err, result) => {
                if (err) {
                    // Retry with orginal SQL
                    orginalSql = orginalSql.replace(/\?/g, function(id) {
                        var pname = '@param' + i;
                        req.input('param' + i, params[i]);
                        i++;
                        return pname;
                    });
                    req.query(orginalSql, (err, result) => {
                        pool.close();
                        if (err) {
                            return cb(null, {
                                success: false,
                                message: err.message
                            });
                        }
                        cb(null, {
                            success: true,
                            metaData: (function(columns) {
                                var fields = [];
                                var col;
                                for ( i in columns ) {
                                    col = columns[i];
                                    col.dbType = col.type.declaration;
                                    col.varType = varTypeMap[col.dbType.toLowerCase()] || 'string';
                                    fields.push(col);
                                }
                                return fields;
                            })(result.recordset.columns)
                        });
                    });
                    return;
                }

                pool.close();
                
                cb(null, {
                    success: true,
                    metaData: (function(columns) {
                        var fields = [];
                        var col;
                        for ( i in columns ) {
                            col = columns[i];
                            col.dbType = col.type.declaration;
                            col.varType = varTypeMap[col.dbType.toLowerCase()] || 'string';
                            fields.push(col);
                        }
                        return fields;
                    })(result.recordset.columns)
                });
            });
        });
    }, 
    getTableNames: function(dbConfig, cb) {
        // To be continued.
        cb(null, []);
    },
    getTableColumns: function(dbConfig, tabName, cb) {
        // To be continued.
        cb(null, []);
    }
}