var express = require( 'express' );
var router = express.Router();
var os = require('os');
var request = require('request');
var requireNocache = require('require-nocache')(module);
var fs = require('fs-extra');
var path = require('path');
var mocker = require('../mock');
var DatabaseService = require('../database-service');
var config = require('../config');
var customInterfacePathBase = '../repository/project/custom-interface/';

router.all('/custom-interface/p/:project_id/call/:interface_name', function(req, res, next) {
    var dbConfiguration;
    try {
        // if (!req.query._d_b_) {
        //     return res.status(500).send('Invalid database config.');
        // }
        dbConfiguration = JSON.parse(req.query._d_b_ || '{"default":"","connections":{}}');
    } catch (e) {
        return next(
            new Error('Invalid database setting for preview.')
        );
    }

    // Mock getDatabaseService
    req.getDatabaseService = function() {
        return new DatabaseService(dbConfiguration);
    };
    
    var serverVars = req.query._s_v_ || '{}';
    try {
        serverVars = JSON.parse(decodeURIComponent(serverVars));
    } catch(e) {
        serverVars = {};
    }

    // Append Session Variables
    var vbs = req.session._variables;
    if (vbs) {
        for (var i in vbs) {
            serverVars[i.toUpperCase()] = vbs[i];
        }
    }
    // Mock getServerVariables
    req.getServerVariables = function() {
        return serverVars;
    };

    // Mock getUser
    req.getUser = function() {
        return {
            getId: function() {
                return serverVars['USER_ID'];
            },
            getName: function() {
                return serverVars['USER_NAME'];
            },
            getRoles: function() {
                return serverVars['ROLES'];
            }
        }
    };

    // Mock enhancer api
    global.Enhancer = mocker.mockEnhancer(dbConfiguration, serverVars, req, res);
    
    res.setHeader('Access-Control-Allow-Origin', '*');

    var pid = req.params.project_id;
    var iname = req.params.interface_name;

    var projectHost = 'https://workbench.enhancer.io';
    if (/previewer\.enhancer\.cc/.test(req.headers.referer)) {
        projectHost = 'http://workbench.enhancer.cc';
    }
    var method = req.method
    if (method == 'OPTIONS') {
        method = req.headers['access-control-request-method'];
    }
    var url = projectHost + '/proj/' + pid + '/httpinterface/name/' + iname + '/' + method;
    var that = this;

    // Get interface definition.
    request({
        url: url,
        headers: {
            'Cookie': 'token=' + req.query._u_t_k_
        }
    }, function(err, r, body) {
        if (err) {
            console.error(err);
            res.status(500)
                .send('Failed to get interface by name \'' + iname + '\'. Reason: ' + err.message);
            return ;
        }
        if (r.statusCode !== 200) {
            console.error(r.statusCode, r.body);
            res.status(500)
                .send('Failed to get interface \'' + iname + '\'. statusCode: ' + r.statusCode);
            return;
        }
        var data, interface;
        try {
            data = JSON.parse(body);
            if (!data.success || !data.result) {
                return res.status(500).send('Failed to get interface \'' + iname + '\'. responseText: ' + body);
            }
        } catch(err) {
            console.log(err);
            return res.status(403).send('Not authorized for ' + iname);
        }

        interface = data.result.interface.replace(/@\w+(\.\w+)*@/g, function(s) {
            return 'Enhancer.getVariable("' + s.split(/@/g)[1].toUpperCase() + '")';
        });

        var file = path.resolve(__dirname, customInterfacePathBase + iname);
        fs.ensureFileSync(file);
        fs.writeFileSync(file, interface);

        var f = requireNocache(customInterfacePathBase + iname);

        if (typeof f !== 'function') {
            return res.status(500)
                .send('[Interface Error] No function module is exported.');
        }
 
        var isCompleted = false;
        res.on('finish', function(e) {
            isCompleted = true;
        });
        setTimeout(function() {
            if (!isCompleted) {
                isCompleted = true;
                res.status(500)
                    .end('[Timeout Error] Interface response timeout, please check if it responses correctly.');
            }
        }, config.isOfficialEnv ? 5000 : (parseInt(data.result.timeout) || 5000));

        try {
            f.apply(that, [req, res, Enhancer]);
        } catch(err) {
            console.log(err);
            isCompleted = true;
            return res.status(500)
                .send('[Interface Calling Error] Error occured when calling interface: '
                    + err.stack);
        }
    });

});

module.exports = router;